#!/bin/sh

set -e

# shellcheck disable=SC2154
if [ -n "$log_level" ]; then
	set -- "$@" --log-level="$log_level"
fi

# shellcheck disable=SC2154
if [ -n "$oidc_issuer_identifier" ]; then
	set -- "$@" --iss="$oidc_issuer_identifier"
fi

# shellcheck disable=SC2154
if [ "$enable_guest_api" = "yes" ]; then
	set -- "$@" --enable-guest-api
fi

if [ "$INSECURE" = "yes" ]; then
	set -- "$@" --insecure
fi

# kwmserver turn
# shellcheck disable=SC2154
if [ -z "$turn_service_url" ]; then
	turn_service_url=https://turnauth.kopano.com/turnserverauth/
fi

if [ -n "$turn_service_url" ]; then
	set -- "$@" --turn-service-url="$turn_service_url"
fi

# shellcheck disable=SC2154
if [ -n "$turn_service_credentials_user" ] && [ -n "$turn_service_credentials_password" ]; then
	turn_service_credentials=/tmp/turn_service_credentials
	echo "$turn_service_credentials_user":"$turn_service_credentials_password" > "$turn_service_credentials"
fi

# shellcheck disable=SC2154
if [ -n "$turn_service_credentials" ]; then
	set -- "$@" --turn-service-credentials="$turn_service_credentials"
fi

# shellcheck disable=SC2154
if [ -n "$turn_server_shared_secret" ]; then
	set -- "$@" --turn-server-shared-secret="$turn_server_shared_secret"
fi

# shellcheck disable=SC2154
if [ -n "$turn_uris" ]; then
	for uri in $turn_uris; do
		set -- "$@" --turn-uri="$uri"
	done
fi

# kwmserver guest
# shellcheck disable=SC2154
if [ "$allow_guest_only_channels" = "yes" ]; then
	set -- "$@" --allow-guest-only-channels
fi

# shellcheck disable=SC2154
if [ -n "$public_guest_access_regexp" ]; then
	set -- "$@" --public-guest-access-regexp="$public_guest_access_regexp"
fi

# shellcheck disable=SC2034
export registration_conf=/kopano/ssl/konnectd-identifier-registration.yaml

# originally I wanted to wait for $registration_conf, but I needed to precreate the file
# so the konnect container (since the startup is running as nobody) can write to it.
exec dockerize \
	-wait http://kopano_konnect:8777/.well-known/openid-configuration \
        -timeout 360s \
	/usr/local/bin/docker-entrypoint.sh serve \
	--registration-conf /kopano/ssl/konnectd-identifier-registration.yaml \
	"$@"
